#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <base64.h>

#define DECODED_BUFFER_SIZE		54
#define ENCODED_BUFFER_SIZE		75

int encode(FILE *in, FILE *out)
{
	char in_buf[DECODED_BUFFER_SIZE];
	char out_buf[ENCODED_BUFFER_SIZE];
	int bytes;

	while ((bytes = fread(in_buf, 1, sizeof(in_buf), in)) > 0)
	{
		bytes = base64_encode(in_buf, bytes, out_buf);
		out_buf[bytes] = '\n';
		fwrite(out_buf, 1, bytes + 1, out);
	}

	return EXIT_SUCCESS;
}

int decode(FILE *in, FILE *out)
{
	char in_buf[ENCODED_BUFFER_SIZE];
	char out_buf[DECODED_BUFFER_SIZE];
	int bytes;

	while (fgets(in_buf, sizeof(in_buf), in))
	{
		bytes = base64_decode(in_buf, strlen(in_buf), out_buf);
		if (bytes < 0)
		{
			fprintf(stderr, "encountered problem during decoding...\n");
			return EXIT_FAILURE;
		}
		fwrite(out_buf, 1, bytes, out);
	}

	return EXIT_SUCCESS;
}

int main(int argc, char *argv[])
{
	FILE *in = stdin, *out = stdout;
	int ret;
	
	if (argc < 2)
	{
		fprintf(stderr, "usage: %s <command> [infile] [outfile]\n\n", argv[0]);
		fprintf(stderr, "options:\n"
			"  enc\tencode stream\n"
			"  dec\tdecode stream\n");

		
		return EXIT_FAILURE;
	}

	if (argc >= 3)
	{
		in = fopen(argv[2], "rb");
		if (!in)
		{
			fprintf(stderr, "couldn't open %s: %s\n", argv[2],
				strerror(errno));
			return EXIT_FAILURE;
		}
	}

	if (argc >= 4)
	{
		out = fopen(argv[3], "wb");
		if (!out)
		{
			fprintf(stderr, "couldn't open %s: %s\n", argv[3],
				strerror(errno));
			return EXIT_FAILURE;
		}
	}

	if (!strcmp(argv[1], "enc"))
		ret = encode(in, out);

	if (!strcmp(argv[1], "dec"))
		ret = decode(in, out);

	if (in != stdin)
		fclose(in);

	if (out != stdout)
		fclose(out);

	return ret;
}

